import tensorflow as tf
from keras.applications import imagenet_utils
from keras.mixed_precision import global_policy


def preprocess_input(inputs, dtype=None):
    if dtype is None:
        dtype = global_policy().compute_dtype

    inputs = tf.cast(inputs, dtype)
    outputs = imagenet_utils.preprocess_input(inputs, data_format='channels_last', mode='torch')

    return outputs



import math
import numpy as np
import tensorflow as tf
from keras import layers
from keras.saving.object_registration import register_keras_serializable
from keras.utils.tf_utils import shape_type_conversion
from tfswin.norm import LayerNorm


@register_keras_serializable(package='TFSwin')
class PatchMerging(layers.Layer):
    def __init__(self, swin_v2=False, **kwargs):
        super().__init__(**kwargs)
        self.input_spec = layers.InputSpec(ndim=4)

        self.swin_v2 = swin_v2

    @shape_type_conversion
    def build(self, input_shape):
        # noinspection PyAttributeOutsideInit
        self.channels = input_shape[-1]
        if self.channels is None:
            raise ValueError('Channel dimensions of the inputs should be defined. Found `None`.')
        self.input_spec = layers.InputSpec(ndim=4, axes={-1: self.channels})

        # Channel permutation after space-to-depth
        # Required due to unusual concatenation order in orignal version
        perm = np.arange(self.channels * 4).reshape((4, -1))
        perm[[1, 2]] = perm[[2, 1]]

        # noinspection PyAttributeOutsideInit
        self.perm = perm.ravel()

        # noinspection PyAttributeOutsideInit
        self.norm = LayerNorm(name='norm')

        # noinspection PyAttributeOutsideInit
        self.reduction = layers.Dense(self.channels * 2, use_bias=False, name='reduction')

        super().build(input_shape)

    def call(self, inputs, *args, **kwargs):
        def _pad(value):
            return None if value is None else value + value % 2

        height_width = tf.shape(inputs)[1:3]
        hpad, wpad = tf.unstack(height_width % 2)
        paddings = [[0, 0], [0, hpad], [0, wpad], [0, 0]]
        outputs = tf.pad(inputs, paddings)
        outputs.set_shape((inputs.shape[0], _pad(inputs.shape[1]), _pad(inputs.shape[2]), inputs.shape[3]))

        outputs = tf.nn.space_to_depth(outputs, 2)
        outputs = tf.gather(outputs, self.perm, batch_dims=-1)

        if self.swin_v2:
            outputs = self.reduction(outputs)
            outputs = self.norm(outputs)
        else:
            outputs = self.norm(outputs)
            outputs = self.reduction(outputs)

        return outputs

    @shape_type_conversion
    def compute_output_shape(self, input_shape):
        def _scale(value):
            return None if value is None else math.ceil(value / 2)

        return input_shape[0], _scale(input_shape[1]), _scale(input_shape[2]), self.channels * 2

    def get_config(self):
        config = super().get_config()
        config.update({'swin_v2': self.swin_v2})

        return config



import tensorflow as tf
from keras.mixed_precision import policy as mixed_precision
from keras.testing_infra import test_combinations, test_utils
from tfswin.norm import LayerNorm


@test_combinations.run_all_keras_modes
class TestLayerNorm(test_combinations.TestCase):
    def setUp(self):
        super(TestLayerNorm, self).setUp()
        self.default_policy = mixed_precision.global_policy()

    def tearDown(self):
        super(TestLayerNorm, self).tearDown()
        mixed_precision.set_global_policy(self.default_policy)

    def test_layer(self):
        test_utils.layer_test(
            LayerNorm,
            kwargs={},
            input_shape=[2, 16, 16, 3],
            input_dtype='float32',
            expected_output_shape=[None, 16, 16, 3],
            expected_output_dtype='float32'
        )

    def test_fp16(self):
        mixed_precision.set_global_policy('mixed_float16')
        test_utils.layer_test(
            LayerNorm,
            kwargs={},
            input_shape=[2, 16, 3],
            input_dtype='float16',
            expected_output_shape=[None, 16, 3],
            expected_output_dtype='float16'
        )


if __name__ == '__main__':
    tf.test.main()



import tensorflow as tf
from keras import layers
from keras.utils.control_flow_util import smart_cond
from keras.saving.object_registration import register_keras_serializable
from keras.utils.tf_utils import shape_type_conversion
from tfswin.drop import DropPath
from tfswin.mlp import MLP
from tfswin.norm import LayerNorm
from tfswin.winatt import WindowAttention


@register_keras_serializable(package='TFSwin')
class SwinBlock(layers.Layer):
    def __init__(self, num_heads, mlp_ratio=4., qkv_bias=True, qk_scale=None, drop=0., attn_drop=0., path_drop=0.,
                 window_pretrain=0, swin_v2=False, **kwargs):
        super().__init__(**kwargs)
        self.input_spec = [
            layers.InputSpec(ndim=4), layers.InputSpec(ndim=0, dtype='int32'), layers.InputSpec(ndim=0, dtype='int32'),
            layers.InputSpec(ndim=1, dtype='int32'), layers.InputSpec(ndim=5)]
        self.num_heads = num_heads
        self.mlp_ratio = mlp_ratio
        self.qkv_bias = qkv_bias
        self.qk_scale = qk_scale
        self.drop = drop
        self.attn_drop = attn_drop
        self.path_drop = path_drop
        self.window_pretrain = window_pretrain
        self.swin_v2 = swin_v2

    @shape_type_conversion
    def build(self, input_shape):
        norm_init = 'zeros' if self.swin_v2 else 'ones'

        # noinspection PyAttributeOutsideInit
        self.norm1 = LayerNorm(gamma_initializer=norm_init, name='norm1')

        # noinspection PyAttributeOutsideInit
        self.attn = WindowAttention(num_heads=self.num_heads, qkv_bias=self.qkv_bias, qk_scale=self.qk_scale,
                                    attn_drop=self.attn_drop, proj_drop=self.drop, window_pretrain=self.window_pretrain,
                                    swin_v2=self.swin_v2, name='attn')

        # noinspection PyAttributeOutsideInit
        self.drop_path = DropPath(self.path_drop)

        # noinspection PyAttributeOutsideInit
        self.norm2 = LayerNorm(gamma_initializer=norm_init, name='norm2')

        # noinspection PyAttributeOutsideInit
        self.mlp = MLP(ratio=self.mlp_ratio, dropout=self.drop, name='mlp')

        super().build(input_shape)

    def call(self, inputs, *args, **kwargs):
        inputs, shift_size, window_size, relative_index, attention_mask = inputs
        height, width = tf.unstack(tf.shape(inputs)[1:3])

        with_shift = tf.greater(shift_size, 0)

        if self.swin_v2:
            outputs = inputs
        else:
            outputs = self.norm1(inputs)

        h_pad = (window_size - height % window_size) % window_size
        w_pad = (window_size - width % window_size) % window_size
        paddings = [[0, 0], [0, h_pad], [0, w_pad], [0, 0]]
        outputs = tf.pad(outputs, paddings)
        padded_height, padded_width = height + h_pad, width + w_pad

        # Cyclic shift
        outputs = smart_cond(
            with_shift,
            lambda: tf.roll(outputs, [-shift_size, -shift_size], [1, 2]),
            lambda: tf.identity(outputs))

        # Partition windows - fused with qkv heads partitioning
        # outputs = window_partition(outputs, padded_height, padded_width, window_size, self.compute_dtype)

        # W-MSA/SW-MSA
        outputs = self.attn([outputs, window_size, relative_index, attention_mask, with_shift])

        # Merge windows - fused with v heads merging
        # outputs = window_reverse(outputs, padded_height, padded_width, window_size, self.compute_dtype)

        # Reverse cyclic shift
        outputs = smart_cond(
            with_shift,
            lambda: tf.roll(outputs, [shift_size, shift_size], [1, 2]),
            lambda: tf.identity(outputs)
        )

        outputs = outputs[:, :height, :width, ...]

        # FFN
        if self.swin_v2:
            outputs = inputs + self.drop_path(self.norm1(outputs))
            outputs += self.drop_path(self.norm2(self.mlp(outputs)))
        else:
            outputs = inputs + self.drop_path(outputs)
            outputs += self.drop_path(self.mlp(self.norm2(outputs)))

        return outputs

    @shape_type_conversion
    def compute_output_shape(self, input_shape):
        return input_shape[0]

    def get_config(self):
        config = super().get_config()
        config.update({
            'num_heads': self.num_heads,
            'mlp_ratio': self.mlp_ratio,
            'qkv_bias': self.qkv_bias,
            'qk_scale': self.qk_scale,
            'drop': self.drop,
            'attn_drop': self.attn_drop,
            'path_drop': self.path_drop,
            'window_pretrain': self.window_pretrain,
            'swin_v2': self.swin_v2
        })

        return config



import tensorflow as tf


def window_partition(inputs, height, width, window_size, dtype=None, name=None):
    with tf.name_scope(name or 'window_partition'):
        inputs = tf.convert_to_tensor(inputs, dtype)

        if 4 != inputs.shape.rank:
            raise ValueError('Expecting inputs rank to be 4.')

        channels = inputs.shape[-1]
        if channels is None:
            raise ValueError('Channel dimensions of the inputs should be defined. Found `None`.')

        windows_height = height // window_size
        windows_width = width // window_size

        outputs = tf.reshape(inputs, [-1, windows_height, window_size, windows_width, window_size, channels])
        outputs = tf.transpose(outputs, [0, 1, 3, 2, 4, 5])
        outputs = tf.reshape(outputs, [-1, window_size ** 2, channels])

        return outputs


def window_reverse(inputs, height, width, window_size, dtype=None, name=None):
    with tf.name_scope(name or 'window_reverse'):
        inputs = tf.convert_to_tensor(inputs, dtype)

        if 3 != inputs.shape.rank:
            raise ValueError('Expecting inputs rank to be 3.')

        channels = inputs.shape[-1]
        if channels is None:
            raise ValueError('Channel dimensions of the inputs should be defined. Found `None`.')

        windows_height = height // window_size
        windows_width = width // window_size

        outputs = tf.reshape(inputs, [-1, windows_height, windows_width, window_size, window_size, channels])
        outputs = tf.transpose(outputs, [0, 1, 3, 2, 4, 5])
        outputs = tf.reshape(outputs, [-1, height, width, channels])

        return outputs


def window_partition_fused(inputs, height, width, window_size, num_heads, qkv_mult=3, dtype=None, name=None):
    with tf.name_scope(name or 'window_partition'):
        inputs = tf.convert_to_tensor(inputs, dtype)

        if 4 != inputs.shape.rank:
            raise ValueError('Expecting inputs rank to be 4.')

        full_channels = inputs.shape[-1]
        if full_channels is None:
            raise ValueError('Channel dimensions of the inputs should be defined. Found `None`.')

        windows_height = height // window_size
        windows_width = width // window_size
        head_channels = full_channels // (qkv_mult * num_heads)

        outputs = tf.reshape(
            inputs, [-1, windows_height, window_size, windows_width, window_size, qkv_mult, num_heads, head_channels])
        outputs = tf.transpose(outputs, [5, 0, 1, 3, 6, 2, 4, 7])
        outputs = tf.reshape(outputs, [qkv_mult, -1, num_heads, window_size ** 2, head_channels])

        return outputs


def window_reverse_fused(inputs, height, width, window_size, num_heads, dtype=None, name=None):
    with tf.name_scope(name or 'window_reverse'):
        inputs = tf.convert_to_tensor(inputs, dtype)

        if 4 != inputs.shape.rank:
            raise ValueError('Expecting inputs rank to be 4.')

        head_channels = inputs.shape[-1]
        if head_channels is None:
            raise ValueError('Channel dimensions of the inputs should be defined. Found `None`.')

        windows_height = height // window_size
        windows_width = width // window_size
        full_channels = head_channels * num_heads

        outputs = tf.reshape(
            inputs, [-1, windows_height, windows_width, num_heads, window_size, window_size, head_channels])
        outputs = tf.transpose(outputs, [0, 1, 4, 2, 5, 3, 6])
        outputs = tf.reshape(outputs, [-1, height, width, full_channels])

        return outputs



import numpy as np
import tensorflow as tf
from keras import backend, layers, models
from keras.applications import imagenet_utils
from keras.utils import conv_utils, data_utils, layer_utils
from tfswin.ape import AbsoluteEmbedding
from tfswin.basic import BasicLayer
from tfswin.embed import PatchEmbedding
from tfswin.merge import PatchMerging
from tfswin.norm import LayerNorm

BASE_URL = 'https://github.com/shkarupa-alex/tfswin/releases/download/{}/swin{}_{}.h5'
WEIGHT_URLS = {
    'swin_tiny_224': BASE_URL.format('3.0.0', '', 'tiny_patch4_window7_224_22k'),
    'swin_small_224': BASE_URL.format('3.0.0', '', 'small_patch4_window7_224_22k'),
    'swin_base_224': BASE_URL.format('3.0.0', '', 'base_patch4_window7_224_22k'),
    'swin_base_384': BASE_URL.format('3.0.0', '', 'base_patch4_window12_384_22k'),
    'swin_large_224': BASE_URL.format('3.0.0', '', 'large_patch4_window7_224_22k'),
    'swin_large_384': BASE_URL.format('3.0.0', '', 'large_patch4_window12_384_22k'),

    'swin2_tiny_256': BASE_URL.format('3.0.0', 'v2', 'tiny_patch4_window16_256'),
    'swin2_small_256': BASE_URL.format('3.0.0', 'v2', 'small_patch4_window16_256'),
    'swin2_base_256': BASE_URL.format('3.0.0', 'v2', 'base_patch4_window12to16_192to256_22kto1k_ft'),
    'swin2_base_384': BASE_URL.format('3.0.0', 'v2', 'base_patch4_window12to24_192to384_22kto1k_ft'),
    'swin2_large_256': BASE_URL.format('3.0.0', 'v2', 'large_patch4_window12to16_192to256_22kto1k_ft'),
    'swin2_large_384': BASE_URL.format('3.0.0', 'v2', 'large_patch4_window12to24_192to384_22kto1k_ft')
}
WEIGHT_HASHES = {
    'swin_tiny_224': '06588c1314ec16f5d46b6381431c1fc9355f82c4eda0f34e7e57ee32048b6d9a',
    'swin_small_224': '49d799e6b860d97377e2d67f866d691f6ee8e02dd6e6d9c485eb3ee0a0076508',
    'swin_base_224': '61792331fce60c47cf7d2bd8e77c8496245cced095a698ab84b275fc20aab32c',
    'swin_base_384': '928ff42574b6c24e55be56563a73419d51e38a0488d72c075c040de8dafd6df2',
    'swin_large_224': '22bf2dc2e5bd99a490b7cd2362519fbe672a576c78c275d12ad17461e75664fd',
    'swin_large_384': '9deaef34b99aa58240cb5f9fa383838e39646e7d23c77942cb3ab320e175124c',

    'swin2_tiny_256': '6416e13f50c46dcd2bf9252d548053fe6443b2a6b4d4bc69cab295923092e3cd',
    'swin2_small_256': '669f81de05bb5fb5419924f1c2847e5ed9ebf2e353ee06699101cc5e124dec4c',
    'swin2_base_256': '50cc998725e9024c9d4da6951f41a6205b4aec104d17f277a77ebf7020074af3',
    'swin2_base_384': '1ba46e24461aab919cb7997cee17de1259d13917d84366a836551d9fe920c6de',
    'swin2_large_256': '87b4cfc35ba6c3fc9de52184c37dfb2ec280e22052730678b1f9c6c4e5719448',
    'swin2_large_384': '9d83698a8cc05eb1ff978665ea8891d5471f8ea40a1b079bf93d241e31306abc'
}


def SwinTransformer(
        pretrain_size, window_size, embed_dim, depths, num_heads, patch_size=4, patch_norm=True, use_ape=False,
        drop_rate=0., mlp_ratio=4., qkv_bias=True, qk_scale=None, attn_drop=0., path_drop=0.1,
        window_pretrain=None, swin_v2=False, model_name='swin', include_top=True, weights=None,
        input_tensor=None, input_shape=None, pooling=None, classes=1000, classifier_activation='softmax'):
    """Instantiates the Swin Transformer architecture.

    Args:
      pretrain_size: height/width of input image for pretraining.
      window_size: window partition size.
      embed_dim: patch embedding dimension.
      depths: depth of each Swin Transformer layer.
      num_heads: number of attention heads.
      patch_size: patch size used to divide input image.
      patch_norm: whether to add normalization after patch embedding.
      use_ape: whether to add absolute position embedding to the patch embedding
      drop_rate: dropout rate.
      mlp_ratio: ratio of mlp hidden units to embedding units.
      qkv_bias: whether to add a learnable bias to query, key, value.
      qk_scale: override default qk scale of head_dim ** -0.5 if set
      attn_drop: attention dropout rate
      path_drop: stochastic depth rate
      model_name: model name.
      include_top: whether to include the fully-connected layer at the top of the network.
      weights: one of `None` (random initialization), 'imagenet' (pre-training on ImageNet or ImageNet 21k), or the
        path to the weights file to be loaded.
      input_tensor: tensor (i.e. output of `layers.Input()`) to use as image input for the model.
      input_shape: shape tuple without batch dimension. Used to create input layer if `input_tensor` not provided.
      pooling: optional pooling mode for feature extraction when `include_top` is `False`.
        - `None` means that the output of the model will be the 3D tensor output of the last layer.
        - `avg` means that global average pooling will be applied to the output of the last layer, and thus the output
          of the model will be a 2D tensor.
        - `max` means that global max pooling will be applied.
      classes: optional number of classes to classify images into, only to be specified if `include_top` is True.
      classifier_activation: the activation function to use on the "top" layer. Ignored unless `include_top=True`.
        When loading pretrained weights, `classifier_activation` can only be `None` or `"softmax"`.

    Returns:
      A `keras.Model` instance.
    """
    if not (weights in {'imagenet', None} or tf.io.gfile.exists(weights)):
        raise ValueError('The `weights` argument should be either `None` (random initialization), `imagenet` '
                         '(pre-training on ImageNet), or the path to the weights file to be loaded.')

    if weights == 'imagenet' and include_top and classes not in {1000, 21841}:
        raise ValueError('If using `weights` as `"imagenet"` with `include_top` as true, '
                         '`classes` should be 1000 or 21841 depending on model type')

    if input_tensor is not None:
        try:
            backend.is_keras_tensor(input_tensor)
        except ValueError:
            raise ValueError(f'Expecting `input_tensor` to be a symbolic tensor instance. '
                             f'Got {input_tensor} of type {type(input_tensor)}')

    if input_tensor is not None:
        tensor_shape = backend.int_shape(input_tensor)[1:]
        if input_shape and tensor_shape != input_shape:
            raise ValueError('Shape of `input_tensor` should equals to `input_shape` if both provided.')
        else:
            input_shape = tensor_shape

    # Determine proper input shape
    input_shape = imagenet_utils.obtain_input_shape(
        input_shape,
        default_size=pretrain_size,
        min_size=32,
        data_format='channel_last',
        require_flatten=False,
        weights=weights)

    if input_tensor is not None:
        if backend.is_keras_tensor(input_tensor):
            image = input_tensor
        else:
            image = layers.Input(tensor=input_tensor, shape=input_shape, dtype='float32')
    else:
        image = layers.Input(shape=input_shape)

    # Define model pipeline
    x = PatchEmbedding(patch_size=patch_size, embed_dim=embed_dim, normalize=patch_norm, name='patch_embed')(image)

    if use_ape:
        pretrain_size_ = conv_utils.conv_output_length(
            pretrain_size, patch_size, padding='same', stride=patch_size, dilation=1)
        x = AbsoluteEmbedding(pretrain_size_)(x)

    x = layers.Dropout(drop_rate, name='pos_drop')(x)

    path_drops = np.linspace(0., path_drop, sum(depths))

    if not swin_v2:
        window_pretrain = np.minimum(window_size, pretrain_size // 2 ** np.arange(2, 6)).tolist()
    elif window_pretrain is None and swin_v2:
        window_pretrain = [0] * len(depths)

    for i in range(len(depths)):
        path_drop = path_drops[sum(depths[:i]):sum(depths[:i + 1])].tolist()
        not_last = i != len(depths) - 1

        x = BasicLayer(depth=depths[i], num_heads=num_heads[i], window_size=window_size, mlp_ratio=mlp_ratio,
                       qkv_bias=qkv_bias, qk_scale=qk_scale, drop=drop_rate, attn_drop=attn_drop, path_drop=path_drop,
                       window_pretrain=window_pretrain[i], swin_v2=swin_v2, name=f'layers.{i}')(x)
        if not_last:
            x = PatchMerging(swin_v2=swin_v2, name=f'layers.{i}/downsample')(x)

    x = LayerNorm(name='norm')(x)

    if include_top or pooling in {None, 'avg'}:
        x = layers.GlobalAveragePooling2D(name='avg_pool')(x)
    elif pooling == 'max':
        x = layers.GlobalMaxPooling2D(name='max_pool')(x)
    else:
        raise ValueError(f'Expecting pooling to be one of None/avg/max. Found: {pooling}')

    imagenet_utils.validate_activation(classifier_activation, weights)
    x = layers.Dense(classes, name='head')(x)
    x = layers.Activation(classifier_activation, dtype='float32', name='pred')(x)

    # Ensure that the model takes into account any potential predecessors of `input_tensor`.
    if input_tensor is not None:
        inputs = layer_utils.get_source_inputs(input_tensor)
    else:
        inputs = image

    # Create model.
    model = models.Model(inputs, x, name=model_name)

    # Load weights.
    if 'imagenet' == weights and model_name in WEIGHT_URLS:
        weights_url = WEIGHT_URLS[model_name]
        weights_hash = WEIGHT_HASHES[model_name]
        weights_path = data_utils.get_file(origin=weights_url, file_hash=weights_hash, cache_subdir='tfswin')
        model.load_weights(weights_path)
    elif weights is not None:
        model.load_weights(weights)

    if include_top:
        return model

    last_layer = 'norm'
    if pooling == 'avg':
        last_layer = 'avg_pool'
    elif pooling == 'max':
        last_layer = 'max_pool'

    outputs = model.get_layer(name=last_layer).output
    model = models.Model(inputs=inputs, outputs=outputs, name=model_name)

    return model


def SwinTransformerTiny224(model_name='swin_tiny_224', pretrain_size=224, window_size=7, embed_dim=96,
                           depths=(2, 2, 6, 2), num_heads=(3, 6, 12, 24), path_drop=0.2, weights='imagenet',
                           classes=21841, **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop,
                           weights=weights, classes=classes, **kwargs)


def SwinTransformerSmall224(model_name='swin_small_224', pretrain_size=224, window_size=7, embed_dim=96,
                            depths=(2, 2, 18, 2), num_heads=(3, 6, 12, 24), path_drop=0.3, weights='imagenet',
                            classes=21841, **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop,
                           weights=weights, classes=classes, **kwargs)


def SwinTransformerBase224(model_name='swin_base_224', pretrain_size=224, window_size=7, embed_dim=128,
                           depths=(2, 2, 18, 2), num_heads=(4, 8, 16, 32), path_drop=0.5, weights='imagenet',
                           classes=21841, **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop,
                           weights=weights, classes=classes, **kwargs)


def SwinTransformerBase384(model_name='swin_base_384', pretrain_size=384, window_size=12, embed_dim=128,
                           depths=(2, 2, 18, 2), num_heads=(4, 8, 16, 32), weights='imagenet', classes=21841, **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, weights=weights, classes=classes,
                           **kwargs)


def SwinTransformerLarge224(model_name='swin_large_224', pretrain_size=224, window_size=7, embed_dim=192,
                            depths=(2, 2, 18, 2), num_heads=(6, 12, 24, 48), weights='imagenet', classes=21841,
                            **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, weights=weights, classes=classes,
                           **kwargs)


def SwinTransformerLarge384(model_name='swin_large_384', pretrain_size=384, window_size=12, embed_dim=192,
                            depths=(2, 2, 18, 2), num_heads=(6, 12, 24, 48), weights='imagenet', classes=21841,
                            **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, weights=weights, classes=classes,
                           **kwargs)


def SwinTransformerV2Tiny256(model_name='swin2_tiny_256', pretrain_size=256, window_size=16, embed_dim=96,
                             depths=(2, 2, 6, 2), num_heads=(3, 6, 12, 24), path_drop=0.2, weights='imagenet',
                             **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop, swin_v2=True,
                           weights=weights, **kwargs)


def SwinTransformerV2Small256(model_name='swin2_small_256', pretrain_size=256, window_size=16, embed_dim=96,
                              depths=(2, 2, 18, 2), num_heads=(3, 6, 12, 24), path_drop=0.3, weights='imagenet',
                              **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop, swin_v2=True,
                           weights=weights, **kwargs)


def SwinTransformerV2Base256(model_name='swin2_base_256', pretrain_size=256, window_size=16, embed_dim=128,
                             depths=(2, 2, 18, 2), num_heads=(4, 8, 16, 32), path_drop=0.2,
                             window_pretrain=(12, 12, 12, 6), weights='imagenet', **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop,
                           window_pretrain=window_pretrain, swin_v2=True, weights=weights, **kwargs)


def SwinTransformerV2Base384(model_name='swin2_base_384', pretrain_size=384, window_size=24, embed_dim=128,
                             depths=(2, 2, 18, 2), num_heads=(4, 8, 16, 32), path_drop=0.2,
                             window_pretrain=(12, 12, 12, 6), weights='imagenet', **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop,
                           window_pretrain=window_pretrain, swin_v2=True, weights=weights, **kwargs)


def SwinTransformerV2Large256(model_name='swin2_large_256', pretrain_size=256, window_size=16, embed_dim=192,
                              depths=(2, 2, 18, 2), num_heads=(6, 12, 24, 48), path_drop=0.2,
                              window_pretrain=(12, 12, 12, 6), weights='imagenet', **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop,
                           window_pretrain=window_pretrain, swin_v2=True, weights=weights, **kwargs)


def SwinTransformerV2Large384(model_name='swin2_large_384', pretrain_size=384, window_size=24, embed_dim=192,
                              depths=(2, 2, 18, 2), num_heads=(6, 12, 24, 48), path_drop=0.2,
                              window_pretrain=(12, 12, 12, 6), weights='imagenet', **kwargs):
    return SwinTransformer(model_name=model_name, pretrain_size=pretrain_size, window_size=window_size,
                           embed_dim=embed_dim, depths=depths, num_heads=num_heads, path_drop=path_drop,
                           window_pretrain=window_pretrain, swin_v2=True, weights=weights, **kwargs)



from pathlib import Path
from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='tfswin',
    version='3.2.3',
    description='Keras (TensorFlow v2) reimplementation of Swin Transformer V1 & V2 models.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/shkarupa-alex/tfswin',
    author='Shkarupa Alex',
    author_email='shkarupa.alex@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=Path("requirements.txt").read_text().splitlines(),
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
